local addonInfo, privateVars = ...

---------- init namespace ---------

if not EnKai then EnKai = {} end
if not EnKai.cooldown then EnKai.cooldown = {} end

local internal    = privateVars.internal
local data        = privateVars.data

---------- init local variables ---------

local _cdManager  = false
local _cooldowns  = {}
local _cdStore    = { ability = {} , item = {} }
local _lastUpdate = nil

---------- local function block ---------

local function _fctProcessAbilityCooldown (_, info)

  local adds, hasAdds = {}, false
  local stops, hasStops = {}, false  

  for key, data in pairs(info) do
    if data == 0 then
      _cdStore.ability[key] = nil
      
      for addon, details in pairs (_cooldowns) do
        for thisKey, _ in pairs(details.ability) do
          if thisKey == key then
            stops[key] = { type = "ability" }
            hasStops = true
          end
        end
      end
      
    else
      _cdStore.ability[key] = { type = "ability", duration = data, start = Inspect.Time.Frame(), remaining = data }
      
      for addon, details in pairs (_cooldowns) do
        for thisKey, _ in pairs(details.ability) do
          if thisKey == key then
            adds[key] = _cdStore.ability[key]
            hasAdds = true
          end
        end
      end
      
    end
  end
  
  if hasAdds == true then EnKai.eventHandlers["EnKai.CooldownManager"]["start"](adds) end
  if hasStops == true then EnKai.eventHandlers["EnKai.CooldownManager"]["stop"](stops) end

end

---------- library public function block ---------

function EnKai.cooldown.init()

  _cooldowns[Inspect.Addon.Current()] = { item = {}, ability = {} }

  if _cdManager == true then return end
  
  if EnKai.internal.checkEvents ("EnKai.CooldownManager", true) == false then return nil end
  
  Command.Event.Attach(Event.Ability.New.Cooldown.Begin , _fctProcessAbilityCooldown, "EnKai.cooldown.Ability.New.Cooldown.Begin")
  Command.Event.Attach(Event.Ability.New.Cooldown.End , _fctProcessAbilityCooldown, "EnKai.cooldown.Ability.New.Cooldown.End")
  
  EnKai.eventHandlers["EnKai.CooldownManager"]["start"], EnKai.events["EnKai.CooldownManager"]["start"] = Utility.Event.Create(addonInfo.identifier, "EnKai.CooldownManager.start")
  EnKai.eventHandlers["EnKai.CooldownManager"]["update"], EnKai.events["EnKai.CooldownManager"]["update"] = Utility.Event.Create(addonInfo.identifier, "EnKai.CooldownManager.update")
  EnKai.eventHandlers["EnKai.CooldownManager"]["stop"], EnKai.events["EnKai.CooldownManager"]["stop"] = Utility.Event.Create(addonInfo.identifier, "EnKai.CooldownManager.stop")
  
  _cdManager = true

end

function EnKai.cooldown.subscribe(type, id)

  if _cooldowns[Inspect.Addon.Current()][type] == nil then
    _cooldowns[Inspect.Addon.Current()][type] = {}
  end
  
  _cooldowns[Inspect.Addon.Current()][type][id] = true

end

function EnKai.cooldown.unsubscribe(type, id)

  _cooldowns[Inspect.Addon.Current()][type][id] = nil

end

---------- addon internal function block ---------

function internal.processCooldowns ()

  if _cdManager == false then return end
  
  local curTime = Inspect.Time.Real()
  
  if _lastUpdate == nil or curTime - _lastUpdate > 1 then
    _lastUpdate = curTime
    local oInspectItemDetail = Inspect.Item.Detail

    local updates, hasUpdates = {}, false
    local start, hasStart = {}, false
    local stop, hasStop = {}, false
    
    for key, details in pairs (_cdStore.ability) do
      
      _cdStore.ability[key].remaining = _cdStore.ability[key].remaining - 1
      
      for addon, details in pairs (_cooldowns) do
        for thisKey, _ in pairs(details.ability) do
          if thisKey == key then
            updates[key] = _cdStore.ability[key]
            hasUpdates = true
          end
        end
      end
    end    
    
    local temp = {}
    
    for addon, details in pairs (_cooldowns) do
        for thisKey, _ in pairs(details.item) do
          
          if temp[thisKey] == nil then
            local flag, details = pcall(oInspectItemDetail, thisKey)
            
            if flag then
            
              if details == nil then
                if _cdStore.item[thisKey] ~= nil then
                  hasStop = true
                  stop[thisKey] = { type = "item" }
                  _cdStore.item[thisKey] = nil
                end
              elseif details.cooldownBegin ~= nil then
                if _cdStore.item[thisKey] == nil then
                  _cdStore.item[thisKey] = { type = "item", duration = details.cooldownDuration, start = details.cooldownBegin, remaining = details.cooldownRemaining }
                  hasStart = true
                  start[thisKey] = _cdStore.item[thisKey]
                else
                  _cdStore.item[thisKey].remaining = details.cooldownRemaining
                  hasUpdates = true
                  updates[thisKey] = _cdStore.item[thisKey]
                end
              else
                if _cdStore.item[thisKey] ~= nil then
                  hasStop = true
                  stop[thisKey] = { type = "item" }
                  _cdStore.item[thisKey] = nil
                end
              end
              
              temp[thisKey] = true
            end
            
          end
          
        end
    end
    
    if hasStop == true then EnKai.eventHandlers["EnKai.CooldownManager"]["stop"](stop) end
    if hasUpdates == true then EnKai.eventHandlers["EnKai.CooldownManager"]["update"](updates) end
    if hasStart == true then EnKai.eventHandlers["EnKai.CooldownManager"]["start"](start) end
    
  end

end